/*
 * yasapi_resample.c
 * Copyright (C) 2015 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>

static HRESULT MediaTypeSetGUID(IMFMediaType *pMediaType, const char *name,
    REFGUID guidKey, REFGUID guidValue, int debug)
{
  HRESULT hr;

  hr=pMediaType->lpVtbl->SetGUID(pMediaType,
    guidKey,                        // [in]  REFGUID guidKey,
    guidValue                       // [in]  REFGUID guidValue
  );

  if (FAILED(hr)) {
    DERROR(debug,E_OUTOFMEMORY,hr);
    DMESSAGEV(debug,"setting guid \"%s\"",name);
    goto set;
  }

  DPRINTF(debug,"    guid \"%s\" set\n",name);
set:
  return hr;
}

static HRESULT MediaTypeSetUINT32(IMFMediaType *pMediaType, const char *name,
    REFGUID guidKey, UINT32 unValue, int debug)
{
  HRESULT hr;

  hr=pMediaType->lpVtbl->SetUINT32(pMediaType,
    guidKey,                    // [in]  REFGUID guidKey,
    unValue                     // [in]  UINT32 unValue
  );

  DPRINTF(debug,"    uint32 \"%s\" set to %u\n",name,unValue);
set:
  return hr;
}

static HRESULT MediaTypeSetWFX(IMFMediaType *pType, WAVEFORMATEX *pwfx,
    int debug)
{
  WAVEFORMATEXTENSIBLE *pwfxx=WAVE_FORMAT_EXTENSIBLE==pwfx->wFormatTag
      ?(WAVEFORMATEXTENSIBLE *)pwfx
      :NULL;
  HRESULT hr;

  // major type
  hr=MediaTypeSetGUID(pType,"major type",&MF_MT_MAJOR_TYPE,
      &MFMediaType_Audio,debug);

  if (FAILED(hr))
    goto attribute;

  // subtype
  if (NULL==pwfxx)
    hr=WAVE_FORMAT_PCM==pwfx->wFormatTag?S_OK:E_FAIL;
  else
    hr=IsEqualGUID(&KSDATAFORMAT_SUBTYPE_PCM,&pwfxx->SubFormat)?S_OK:E_FAIL;

  if (FAILED(hr)) {
    DMESSAGE(debug,"format not integer pcm");
    goto attribute;
  }

  hr=MediaTypeSetGUID(pType,"subtype",&MF_MT_SUBTYPE,
      &MFAudioFormat_PCM,debug);

  if (FAILED(hr))
    goto attribute;

  // channels
  hr=MediaTypeSetUINT32(pType,"channels",&MF_MT_AUDIO_NUM_CHANNELS,
      pwfx->nChannels,debug);

  if (FAILED(hr))
    goto attribute;

  // sample rate
  hr=MediaTypeSetUINT32(pType,"sample rate",&MF_MT_AUDIO_SAMPLES_PER_SECOND,
      pwfx->nSamplesPerSec,debug);

  if (FAILED(hr))
    goto attribute;

  // block alignment
  hr=MediaTypeSetUINT32(pType,"block alignment",&MF_MT_AUDIO_BLOCK_ALIGNMENT,
      pwfx->nBlockAlign,debug);

  if (FAILED(hr))
    goto attribute;

  // bitrate
  hr=MediaTypeSetUINT32(pType,"bitrate",&MF_MT_AUDIO_AVG_BYTES_PER_SECOND,
      pwfx->nAvgBytesPerSec,debug);

  if (FAILED(hr))
    goto attribute;

  // bps
  hr=MediaTypeSetUINT32(pType,"bps",&MF_MT_AUDIO_BITS_PER_SAMPLE,
      pwfx->wBitsPerSample,debug);

  if (FAILED(hr))
    goto attribute;

  // independent
  hr=MediaTypeSetUINT32(pType,"independent",&MF_MT_ALL_SAMPLES_INDEPENDENT,
      TRUE,debug);

  if (FAILED(hr))
    goto attribute;

  if (NULL!=pwfxx) {
    // channel mask
    hr=MediaTypeSetUINT32(pType,"channel mask",&MF_MT_AUDIO_CHANNEL_MASK,
        pwfxx->dwChannelMask,debug);

    if (FAILED(hr))
      goto attribute;

    // valid bps
    hr=MediaTypeSetUINT32(pType,"valid bps",&MF_MT_AUDIO_VALID_BITS_PER_SAMPLE,
        pwfxx->Samples.wValidBitsPerSample,debug);

    if (FAILED(hr))
      goto attribute;
  }
// cleanup:
attribute:
  return hr;
}

HRESULT MediaTypeCreateWFX(IMFMediaType **ppMediaType, WAVEFORMATEX *pwfx,
    const char *name, int debug)
{
  IMFMediaType *pMediaType;
  HRESULT hr;

  // create.
  hr=MFCreateMediaType(
    &pMediaType                   // IMFMediaType **ppMFType
  );
  
  if (FAILED(hr)) {
    DMESSAGEV(debug,"creating %s type",name);
    goto type;
  }

  DPRINTF(debug,"  %s type created\n",name);

  // set attributes.
  hr=MediaTypeSetWFX(pMediaType,pwfx,debug);

  if (FAILED(hr))
    goto attribute;

  *ppMediaType=pMediaType;

  return hr;
attribute:
  pMediaType->lpVtbl->Release(pMediaType);
type:
  return hr;
}
