/*
 * yasapi_player_push.c
 * Copyright (C) 2015 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>

///////////////////////////////////////////////////////////////////////////////
static HRESULT PlayerInitializeAudioClientPush(Player *pPlayer,
    REFERENCE_TIME hnsDevicePeriod)
{
  IAudioClient *pAudioClient=pPlayer->pAudioClient;
  AUDCLNT_SHAREMODE eShareMode=pPlayer->pPlugin->pOptions->eShareMode;
  WAVEFORMATEX *pwfx=&pPlayer->wfxx.Format;

  return pAudioClient->lpVtbl->Initialize(pAudioClient,
    eShareMode,           // [in]  AUDCLNT_SHAREMODE ShareMode,
    0,                    // [in]  DWORD StreamFlags,
    hnsDevicePeriod,      // [in]  REFERENCE_TIME hnsBufferDuration,
    0,                    // [in]  REFERENCE_TIME hnsPeriodicity,
    pwfx,                 // [in]  const WAVEFORMATEX *pFormat,
    NULL                  // [in]  LPCGUID AudioSessionGuid
  );
}

static int PlayerCreateEventPush(Player *pPlayer)
{
  pPlayer->hEvent=NULL;

  return 0;
}

static void PlayerDestroyEventPush(Player *pPlayer)
{
}

static BOOL PlayerCanReadPush(Player *pPlayer)
{
#if defined (YASAPI_DEBUG) // {
  int debug=pPlayer->pPlugin->pOptions->debug;

  if (0==pPlayer->ring.dwWritten)
    DPUTS(debug,"Warning: underflow\n");
#endif // }

  return 0<pPlayer->ring.dwWritten;
}

HRESULT PlayerGetFramesWritePush(Player *pPlayer, UINT32 *pNumFramesWrite,
    UINT32 *pNumFramesPadding)
{
#if defined (YASAPI_DEBUG) // {
  int debug=pPlayer->pPlugin->pOptions->debug;
#else // } {
  int debug=0;
#endif // }
  IAudioClient *pAudioClient=pPlayer->pAudioClient;
  Ring *pRing=&pPlayer->ring;
  WAVEFORMATEX *pwfx=&pPlayer->wfxx.Format;
  UINT32 nFrames=pPlayer->nFrames;
  UINT32 nFramesPadding;
  UINT32 nFramesWrite;
  UINT32 nFramesRing;
  HRESULT hr;

  // get padding.
  hr=pAudioClient->lpVtbl->GetCurrentPadding(pAudioClient,
    &nFramesPadding       // [out]  UINT32 *pNumPaddingFrames
  );

  if (FAILED(hr)) {
    DERROR(debug,AUDCLNT_E_NOT_INITIALIZED,hr);
    DERROR(debug,AUDCLNT_E_DEVICE_INVALIDATED,hr);
    DERROR(debug,AUDCLNT_E_SERVICE_NOT_RUNNING,hr);
    DERROR(debug,E_POINTER,hr);
    DMESSAGE(debug,"getting padding");
    goto padding;
  }

  // caclulate the number of frames to render.
  nFramesWrite=nFrames-nFramesPadding;
  nFramesRing=pRing->dwWritten/pwfx->nBlockAlign;

  if (nFramesWrite>nFramesRing)
    nFramesWrite=nFramesRing;

  *pNumFramesWrite=nFramesWrite;
  *pNumFramesPadding=nFramesPadding;
padding:
  return hr;
}

static int PlayerWaitPush(Player *pPlayer, LONGLONG time)
{
  TimerSleep(&pPlayer->pPlugin->timer,time);

  return 0;
}

const PlayerStrategy *PlayerPush(void)
{
  static int initialized;
  static PlayerStrategy strategy;

  if (0==initialized) {
    strategy.name="push";
    strategy.InitializeAudioClient=PlayerInitializeAudioClientPush;
    strategy.CreateEvent=PlayerCreateEventPush;
    strategy.DestroyEvent=PlayerDestroyEventPush;
    strategy.CanRead=PlayerCanReadPush;
    strategy.Wait=PlayerWaitPush;
    strategy.GetFramesWrite=PlayerGetFramesWritePush;
    initialized=1;
  }

  return &strategy;
}
